<?php

namespace Drupal\feeds_filemaker\Feeds\Parser;

use Drupal\feeds\FeedInterface;
use Drupal\feeds\Plugin\Type\Parser\ParserInterface;
use Drupal\feeds\Result\FetcherResultInterface;
use Drupal\feeds\Result\ParserResult;
use Drupal\feeds\Plugin\Type\PluginBase;
use Drupal\feeds\StateInterface;
use Drupal\feeds_filemaker\Feeds\Item\FileMakerItem;

/**
 * Defines a base FileMaker JSON parser.
 */
abstract class FileMakerJsonParserBase extends PluginBase implements ParserInterface {

  /**
   * {@inheritdoc}
   */
  public function parse(FeedInterface $feed, FetcherResultInterface $fetcher_result, StateInterface $state): ParserResult {
    $raw = $fetcher_result->getRaw();
    $parsed_data = json_decode($raw, TRUE);
    $result = new ParserResult();
    $skipped_items_count = 0;

    if (isset($parsed_data)) {
      foreach ($parsed_data as $item) {
        if (!$this->shouldSkipItem($item)) {
          $result->addItem($this->parseItem($item));
        }
        else {
          $skipped_items_count++;
        }
      }
    }

    // Add a message about skipped items
    if ($skipped_items_count > 0) {
      $state->setMessage($this->t('@count items were skipped during import.', [
        '@count' => $skipped_items_count,
      ]));
    }

    return $result;
  }

  /**
   * Determines whether an item should be skipped during parsing.
   *
   * @param array $item
   *   The item to be checked.
   *
   * @return bool
   *   TRUE if the item should be skipped, FALSE otherwise.
   */
  protected function shouldSkipItem(array $item): bool {
    // Default implementation: don't skip any items.
    return FALSE;
  }

  abstract protected function parseItem(array $data): FileMakerItem;

  abstract public function getMappingSources(): array;

  abstract public function getSupportedCustomSourcePlugins(): array;

}
